//======================================================== file = bridge.c ====
//=  Program to create a bridge in a computer with 2 interfaces. It will      =
//=  receive in one interface and send through the other one. Based on demo.c =
//=============================================================================
/*
 * Copyright (c) 1999 - 2003
 * NetGroup, Politecnico di Torino (Italy)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the Politecnico di Torino nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
//=============================================================================
//=  Notes:                                                                   =
//=    1) This product includes software developed by the Politecnico         =
//=        di Torino, and its contributors.                                   = 
//=    2) Runs on WinXP and WinVista only. A device index has to be defined   =
//=       using the definition of DEVICE_INDEX on this file.                  =
//=    3) A warning is shown when compiling which doesn't affect performance  =
//=    4) Is assumed that 2 interfaces are available to send  receive         =
//=---------------------------------------------------------------------------=
//= Example execution:                                                        =
//= Initializing all interfaces ...                                           =
//= -> Opening rpcap://\Device\NPF_GenericDialupAdapter                       =
//= -> Opening rpcap://\Device\NPF_{5F574AE2-F3E7-40CA-868E-32B06832B705}     =
//= Receiving through 1 and Sending through 2 ...                             =
//=  Received packet from 1 (first 40 bytes of 60) = FF-FF-FF-FF-FF-FF-00-07- =
//=  E9-5E-DC-53-08-06-00-01-08-00-06-04-00-01-00-0                           =
//= 7-E9-5E-DC-53-83-F7-02-0F-00-00-00-00-00-00-83-F7-                        =
//=  Sending through device 2                                                 =
//=---------------------------------------------------------------------------=
//=  Build:                                                                   =
//=  gcc bridge.c rawstuff.c -lwpcap -lws2_32 -L.\Lib -I.\Include -o bridge   =
//=---------------------------------------------------------------------------=
//=  Execute: bridge                                                          =
//=---------------------------------------------------------------------------=
//=  Authors: Miguel Jimeno                                                   =
//=           Zornitza Genova                                                 =
//=           Kenneth J. Christensen                                          =
//=           University of South Florida                                     =
//=           WWW: http://www.csee.usf.edu/~christen                          =
//=           Email: christen@csee.usf.edu                                    =
//=---------------------------------------------------------------------------=
//=  History: MJ/KJC (01/24/09) - Genesis from demo.c                         =
//=============================================================================
//----- Defines ---------------------------------------------------------------
#define MAX_PKT_LEN   1514   // Maximum packet length
#define NUM_BYTES       40   // Number of bytes to output
#define INTER_RECV       3   // Interface from which packets will be received
#define INTER_SEND       6   // Interface to which packets will be sent

//----- Include files ---------------------------------------------------------
#include "pcap.h"        // Winpcap libraries
#include "remote-ext.h"  // Winpcap libraries

//----- Function prototypes ---------------------------------------------------
// Initialize interfaces
int  init_(void);                                          
// Receive a raw packet
int  recv_(int numInt, int *pkt_len);                      
// Send a raw packet
int  send_(int numInt, unsigned char *packet, int pkt_len);
// Close all interfaces
void close_(void);                                         

//----- Global variables ------------------------------------------------------
unsigned char *recv_buf;     // Global packet receive buffer
//=============================================================================
//=  Main program                                                             =
//=============================================================================
void main(void)
{
  int           packet_len;              // Packet length
  int           retcode;                 // Return code
  int           i;                       // Loop counter

  // Initialize the interfaces
  printf("Initializing all interfaces ... \n");
  retcode = init_();
  if (retcode < 0)
  {
    printf("*** ERROR - init_() failed (retcode = %d) \n", retcode);
    exit(-1);
  }

  printf("Receiving through %d and Sending through %d ... \n", 
         INTER_RECV, INTER_SEND);

  while (1)
  {
    //Receiving interface INTER_RECV
    retcode = recv_(INTER_RECV, &packet_len);
    if (retcode == 0)
    {
      printf("*** WARNING - recv_() found nothing in buffers \n");
      exit(-1);
    }
    if (retcode < 0)
    {
      printf("*** ERROR - recv_() failed (retcode = %d) \n", retcode);
      exit(-1);
    }

    if (packet_len > 0)// Do something only if there are bytes received
    {
      // Output the received packet if receive retcode > 0
      if (retcode > 0)
      {
        printf("  Received packet from %d (first %d bytes of %d) = ", INTER_RECV,
          NUM_BYTES, packet_len);
        for (i=0; i<NUM_BYTES; i++)
          printf("%02X-", recv_buf[i]);
        printf("\n");
      }

     // Send received buffer through interface INTER_SEND
      printf("  Sending through device %d \n", INTER_SEND);
      retcode = send_(INTER_SEND, recv_buf, packet_len);
      if (retcode < 0)
      {
        printf("*** ERROR - send_() failed (retcode = %d) \n", retcode);
        exit(-1);
      }
    }//End if more than 0 bytes are received    
  }//End while loop
  // Free memory for receive buffer
  free(recv_buf);

  // Close the interfaces
  printf("Closing all interfaces...\n");
  close_();
}
