//====================================================== file = rawstuff.c ====
//=  Functions for raw init, send, recv, and close                            =
//=============================================================================
// Copyright (c) 1999 - 2003
// NetGroup, Politecnico di Torino (Italy)
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
// notice, this list of conditions and the following disclaimer in the
// documentation and/or other materials provided with the distribution.
// 3. Neither the name of the Politecnico di Torino nor the names of its
// contributors may be used to endorse or promote products derived from
// this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//=============================================================================
//=  Notes:                                                                   =
//=    1) This product includes software developed by the Politecnico         =
//=       di Torino, and its contributors.                                    =
//=============================================================================
//=  Contains:                                                                =
//=    init_()             - Raw initialization of all interfaces             =
//=    send_()             - Raw send                                         =
//=    recv_()             - Raw receive                                      =
//=    close_()            - Raw close                                        =
//=    int numInterfaces() - Count number of interfaces                       =
//=---------------------------------------------------------------------------=
//=  History:  ZG/KJC (10/02/01) - Genesis                                    =
//=            JNS    (03/13/02) - Hang on receive bug fixed                  =
//=            JNS    (03/13/02) - Set-up for promiscuous mode in init_()     =
//=            MJ     (01/28/09) - Modified for new version of Winpcap        =
//=============================================================================
//----- Include files ---------------------------------------------------------
#include <stdio.h>       // Needed for printf()
#include "pcap.h"        // Winpcap libraries
#include "remote-ext.h"  // Winpcap libraries

//----- Defines ---------------------------------------------------------------
#define MAX_PKT_LEN                             1514   // Max packet length
#define SIZE                                     512   // Adapter_name size
#define MAX_PKT_BLOCK                             50   // Max packets in block
#define RECV_BUFF_SIZE   MAX_PKT_BLOCK * MAX_PKT_LEN   // Receive buffer size

// ---- Globals ---------------------------------------------------------------
struct Interfaces                    // Structure with all interfaces
{
  pcap_t     *hdl;                   // Handler for the interface
};
struct Interfaces* ListHandlers;     // List of interfaces
int    MaxNumInt;                    // Number of interfaces available
extern unsigned char *recv_buf;      // Global packet receive buffer

//----- Function prototypes ---------------------------------------------------
int numInterfaces(void);             // Function to count number of interfaces

//=============================================================================
//= Function to initialize all the interfaces listed in listHandlers          =
//=  - Returns 1 if success and -1 if fails in at least one interface         =
//=============================================================================
int init_(void)
{
  struct bpf_program fcode;                     //
  pcap_if_t *alldevs;                           // List of all interfaces
  pcap_if_t *d;                                 // Interface to be open
  int       i;                                  // Counter
  char      errbuf[PCAP_ERRBUF_SIZE];           // Buffer to store an error
  u_int     netmask;                            // netmask of the interface

  // The Packet filter (exactly as Wireshark does) is set in the following line
  char packet_filter[] = "";

  // Retrieve the number of available interfaces for this computer
  MaxNumInt = numInterfaces();

  // Malloc memory for list of interfaces
  ListHandlers = (struct Interfaces*)malloc(MaxNumInt *
                  sizeof(struct Interfaces*));
  //exit with error if memory could not be allocated
  if (ListHandlers == NULL)
    return (-1);

  // Retrieve the interfaces list
  if (pcap_findalldevs_ex(PCAP_SRC_IF_STRING, NULL, &alldevs, errbuf) == -1)
  {
    fprintf(stderr,"*** ERROR - in pcap_findalldevs: %s\n", errbuf);
    return (-1);
  }

  // Open all interfaces
  i = 0;
  for(d=alldevs; d; d=d->next)
  {
    printf("-> Opening %s\n", d->name);

    // Open the interface
    if ( (ListHandlers[i].hdl = pcap_open(d->name,  // name of the interface
                 65536,     // portion of the packet to capture.
                            // 65536 grants capture of the whole packet.
                 PCAP_OPENFLAG_PROMISCUOUS,         // promiscuous mode
                 1000,      // read timeout
                 NULL,      // remote authentication
                 errbuf     // error buffer
                 ) ) == NULL)
    {
      fprintf(stderr,
        "\n*** ERROR - Unable to open adapter. %s not supported by WinPcap\n");
      pcap_freealldevs(alldevs);
      return (-1);
    }

    // Check the link layer. We support only Ethernet for simplicity.
    if(pcap_datalink(ListHandlers[i].hdl) != DLT_EN10MB)
    {
      fprintf(stderr,
        "\n*** ERROR - This program works only on Ethernet networks.\n");
      pcap_freealldevs(alldevs);
      return (-1);
    }

    // If the interface has an address, get its net mask
    if(d->addresses != NULL)
      // Retrieve the mask of the first address of the interface
      netmask=((struct sockaddr_in *)(d->addresses->netmask))->sin_addr.S_un.S_addr;
    else
      // If the interface is without addresses suppose to be in a C class net.
      netmask=0xffffff;

    // Compile the filter
    if (pcap_compile(ListHandlers[i].hdl, &fcode, packet_filter, 1, netmask) < 0)
    {
      fprintf(stderr,
        "\n*** ERROR - Unable to compile the packet filter. Check the syntax.");
      pcap_freealldevs(alldevs);
      return (-1);
    }

    // Set the filter
    if (pcap_setfilter(ListHandlers[i].hdl, &fcode)<0)
    {
      fprintf(stderr,
        "\n*** ERROR - Error setting the filter.\n");
      pcap_freealldevs(alldevs);
      return (-1);
    }
    //Increment counter for number of interfaces (used as index in ListHandlers)
    i++;
  }

  // At this point, we don't need any more the interfaces list. Free it
  pcap_freealldevs(alldevs);
  return (1);
}


//=============================================================================
//= Function to send a packet                                                 =
//=  - Receives interface index, packet and packet length. Returns 1 if no    =
//=    error or -1 if there is an error.                                      =
//=============================================================================
int send_(int numInt, unsigned char *packet, int pkt_len)
{
  // Verify the interface number is less than MaxNumInt or return with error
  if (numInt > MaxNumInt)
    return (-1);

  // Send down the frame
  if (pcap_sendpacket(ListHandlers[numInt-1].hdl, packet, pkt_len) != 0)
  {
    fprintf(stderr,"\n*** ERROR - Error sending through %d: %s\n",
      numInt, pcap_geterr(ListHandlers[numInt-1].hdl));
    return (-1);
  }

  // Return with success
  return(1);
}

//=============================================================================
//= Function to receive a packet                                              =
//=  - receives interface index, and packet length. Returns 1 if no error, -1 =
//=    if there is an error, 0 if it times out.                               =
//=============================================================================
int recv_(int numInt, int *pkt_len)
{
  int res;                             // Response variable
  struct pcap_pkthdr *header;          // header of packet
  u_char *pkt_data;                    // the data of the packet

  // Verify the interface number is less than MaxNumInt or return with error
  if (numInt > MaxNumInt)
    return (-1);

  // Retrieve the frame
  res = pcap_next_ex(ListHandlers[numInt-1].hdl, &header, &pkt_data);
  if (res < 0) // Error reading
  {
    fprintf(stderr,"\n*** ERROR - Error receiving: %s\n",
      pcap_geterr(ListHandlers[numInt-1].hdl));
    return (-1);
  }
  if (res == 0) // Timeout elapsed (no packet to send)
  {
    *pkt_len  = 0;
    return (0);
  }

  // Otherwise there is something to read
  *pkt_len  = header->len;
  recv_buf = pkt_data;

  // Return with success
  return(1);
}

//=============================================================================
//= Function to close all the interfaces                                        =
//=  - Returns nothing                                                        =
//=============================================================================
void close_(void)
{
  int i;                            // Counter

  // Close the interfaces
  for (i=0; i<MaxNumInt; i++)
    pcap_close(ListHandlers[i].hdl);
}

//=============================================================================
//= Function to count number of interfaces available in this computer         =
//=  - Returns number of interfaces available                                 =
//=============================================================================
int numInterfaces()
{
  pcap_if_t *alldevs;               // list of interfaces
  pcap_if_t *d;                     // interface handler
  int i;                            // counter for number of interfaces
  char errbuf[PCAP_ERRBUF_SIZE];    // buffer to store error messages

  // Retrieve the interfaces list or terminate program if there is an error
  if (pcap_findalldevs_ex(PCAP_SRC_IF_STRING, NULL, &alldevs, errbuf) == -1)
  {
    fprintf(stderr,"*** ERROR - Error in pcap_findalldevs: %s\n", errbuf);
    return (-1);
  }

  i = 0; // Initialize counter
  for(d=alldevs; d; d=d->next)
    ++i; // Increment number of interfaces available

  // At this point, we don't need any more the interfaces list. Free it
  pcap_freealldevs(alldevs);

  // return number of interfaces
  return (i);
}
